@@ -115,6 +115,17 @@ sub config_file {
   }
 }
 
+sub compiler {
+  my $self = shift;
+  if (@_) {
+    $self->{compiler} = $_[0];
+    return $self;
+  }
+  else {
+    return $self->{compiler};
+  }
+}
+
 sub runtime {
   my $self = shift;
   if (@_) {
@@ -185,6 +196,12 @@ sub new {
   
   $self->{builder} = $builder;
   
+  # Compile SPVM
+  my $compiler = SPVM::Builder::Compiler->new(
+    include_dirs => $builder->include_dirs
+  );
+  $self->{compiler} = $compiler;
+  
   return bless $self, $class;
 }
 
@@ -356,10 +373,8 @@ sub compile {
   # Module name
   my $module_name = $self->{module_name};
   
-  # Compile SPVM
-  my $compiler = SPVM::Builder::Compiler->new(
-    include_dirs => $builder->include_dirs
-  );
+  my $compiler = $self->compiler;
+  
   my $success = $compiler->compile($module_name, __FILE__, __LINE__);
   unless ($success) {
     $compiler->print_error_messages(*STDERR);
@@ -529,9 +544,9 @@ EOS
     }
   }
 
-  $source .= "static int32_t* SPVM_BOOTSTRAP_create_bootstrap_set_native_method_addresses(SPVM_ENV* env);\n";
+  $source .= "static int32_t* SPVM_BOOTSTRAP_create_bootstrap_set_native_method_addresses(SPVM_ENV* env);\n\n";
 
-  $source .= "static int32_t* SPVM_BOOTSTRAP_get_runtime_codes();\n";
+  $source .= "static void* SPVM_BOOTSTRAP_build_runtime(SPVM_ENV* env, void* compiler);\n\n";
 
   $source .= <<"EOS";
 static int32_t* SPVM_BOOTSTRAP_set_native_method_address(SPVM_ENV* env, const char* module_name, const char* method_name, void* native_address) {
@@ -569,7 +584,25 @@ int32_t main(int32_t command_args_length, const char *command_args[]) {
   _setmode(fileno(stdin), _O_BINARY);
 #endif
   
-  SPVM_ENV* env = SPVM_NATIVE_new_env_prepared();
+  // Create env
+  SPVM_ENV* env = SPVM_NATIVE_new_env_raw();
+  
+  // Compiler
+  void* compiler = env->api->compiler->new_instance();
+  
+  void* runtime = SPVM_BOOTSTRAP_build_runtime(env, compiler);
+  
+  // Set runtime
+  env->runtime = runtime;
+  
+  // Free compiler
+  env->api->compiler->free_instance(compiler);
+  
+  // Set precompile method addresses
+  SPVM_BOOTSTRAP_create_bootstrap_set_precompile_method_addresses(env);
+  
+  // Set native method addresses
+  SPVM_BOOTSTRAP_create_bootstrap_set_native_method_addresses(env);
   
   SPVM_VALUE* stack = env->new_stack(env);
   
@@ -661,81 +694,78 @@ EOS
   return $source;
 }
 
-sub create_bootstrap_get_runtime_codes_func_source {
+sub create_bootstrap_build_runtime_source {
   my ($self) = @_;
-
+  
   # Builder
   my $builder = $self->builder;
-
-  my $runtime_codes = SPVM::Builder::Runtime->get_runtime_codes($self->runtime);
-  my $runtime_codes_length = @$runtime_codes;
+  
   my $source = '';
   
-  my $runtime_codes_str = join(",", @$runtime_codes);
-
-  $source .= "static int32_t SPVM_BOOTSTRAP_runtime_codes[$runtime_codes_length] = {$runtime_codes_str};\n";
-
   $source .= <<"EOS";
-static int32_t* SPVM_BOOTSTRAP_get_runtime_codes() {
-  return SPVM_BOOTSTRAP_runtime_codes;
-}
+static void* SPVM_BOOTSTRAP_build_runtime(SPVM_ENV* env, void* compiler) {
+  
 EOS
   
-  return $source;
-}
-
-sub create_bootstrap_new_env_prepared_func_source {
-  my ($self) = @_;
-  
-  # Builder
-  my $builder = $self->builder;
-  
-  # Module name
-  my $module_name = $self->module_name;
-  
-  # Module names
   my $module_names = $self->get_module_names;
   
-  my $source = '';
+  my $compiler = $self->compiler;
   
-  my $set_precompile_method_addresses_source = '';
+  for my $module_name (@$module_names) {
+    my $module_file = $compiler->get_module_file($module_name);
+    
+    my $source_module_file = '';
+    
+    $source_module_file .= qq|  {\n|;
+
+    $source_module_file .= qq|    void* module_file = env->api->module_file->new_instance(compiler);\n|;
+    
+    $source_module_file .= qq|    env->api->module_file->set_module_name(compiler, module_file, "$module_name");\n|;
+    
+    if (defined $module_file->{rel_file}) {
+      $source_module_file .= qq|    env->api->module_file->set_rel_file(compiler, module_file, "$module_file->{rel_file}");\n|;
+    }
+    
+    if (defined $module_file->{content}) {
+      my $content_espcaped = $module_file->{content};
+      
+      {
+        use bytes;
+        $content_espcaped =~ s/\\/\\\\/g;
+        $content_espcaped =~ s/"/\\"/g;
+        $content_espcaped =~ s/([^[:print:]])/sprintf("\\%03o", ord($1))/ge;
+      }
+      
+      $source_module_file .= qq|    env->api->module_file->set_content(compiler, module_file, "$content_espcaped");\n|;
+    }
+    
+    $source_module_file .= qq|    env->api->module_file->set_content_length(compiler, module_file, $module_file->{content_length});\n|;
+      
+    $source_module_file .= qq|    env->api->compiler->add_module_file(compiler, "$module_name", module_file);\n\n|;
+    
+    $source_module_file .= qq|  }\n|;
+    
+    $source .= $source_module_file;
+  }
+  
+  $source .= qq|  env->api->compiler->set_start_file(compiler, __FILE__);\n|;
+  
+  $source .= qq|  env->api->compiler->set_start_line(compiler, __LINE__ + 1);\n|;
+  
+  my $start_module_name = $self->{module_name};
+  
+  $source .= qq|  int32_t error_id = env->api->compiler->compile(compiler, \"$start_module_name\");\n|;
+  
+  $source .= qq|  if (error_id != 0) {\n|;
+  $source .= qq|    fprintf(stderr, "[Unexpected Compile Error]%s.", env->api->compiler->get_error_message(compiler, 0));\n|;
+  $source .= qq|    exit(255);\n|;
+  $source .= qq|  }\n|;
+  
+  $source .= qq|  void* runtime = env->api->compiler->build_runtime(compiler);\n|;
+  
+  $source .= qq|  return runtime;\n|;
   
-  my $config_exe = $self->config;
-  $set_precompile_method_addresses_source = "SPVM_BOOTSTRAP_create_bootstrap_set_precompile_method_addresses(env);";
   $source .= <<"EOS";
-SPVM_ENV* SPVM_NATIVE_new_env_prepared() {
-EOS
-
-  $source .= <<"EOS";
-
-  // Create env
-  SPVM_ENV* env = SPVM_NATIVE_new_env_raw();
-
-  // New runtime
-  void* runtime = env->api->runtime->new_instance();
-  
-  // Runtime allocator
-  void* runtime_allocator = env->api->runtime->get_allocator(runtime);
-  
-  // Create SPVM 32bit codes
-  int32_t* runtime_codes = SPVM_BOOTSTRAP_get_runtime_codes();
-  
-  // Build runtime
-  env->api->runtime->build(runtime, runtime_codes);
-
-  // Set runtime information
-  env->runtime = runtime;
-
-  // Initialize env
-  env->init_env(env);
-
-  // Set precompile method addresses
-  $set_precompile_method_addresses_source
-  
-  // Set native method addresses
-  SPVM_BOOTSTRAP_create_bootstrap_set_native_method_addresses(env);
-  
-  return env;
 }
 EOS
   
@@ -840,20 +870,16 @@ sub create_bootstrap_source {
     
     # main function
     $bootstrap_source .= $self->create_bootstrap_main_func_source;
-
-    # SPVM_NATIVE_new_env_prepared function
-    $bootstrap_source .= $self->create_bootstrap_new_env_prepared_func_source;
-
+    
     # Set precompile method addresses function
     my $config_exe = $self->config;
     $bootstrap_source .= $self->create_bootstrap_set_precompile_method_addresses_func_source;
-
+    
     # Set native method addresses function
     $bootstrap_source .= $self->create_bootstrap_set_native_method_addresses_func_source;
-
-    # get_runtime_codes function
-    $bootstrap_source .= $self->create_bootstrap_get_runtime_codes_func_source;
-
+    
+    $bootstrap_source .= $self->create_bootstrap_build_runtime_source;
+    
     # Build source directory
     my $build_src_dir = SPVM::Builder::Util::create_build_src_path($self->builder->build_dir);
     mkpath $build_src_dir;
@@ -861,7 +887,7 @@ sub create_bootstrap_source {
     
     open my $bootstrap_source_fh, '>', $bootstrap_source_file
       or die "Can't open file $bootstrap_source_file:$!";
-
+    
     print $bootstrap_source_fh $bootstrap_source;
   };
   
