@@ -9,14 +9,14 @@ use File::Basename 'dirname';
 use overload bool => sub {1}, '""' => sub { shift->to_string }, fallback => 1;
 
 # Fields
-sub class_name {
+sub module_name {
   my $self = shift;
   if (@_) {
-    $self->{class_name} = $_[0];
+    $self->{module_name} = $_[0];
     return $self;
   }
   else {
-    return $self->{class_name};
+    return $self->{module_name};
   }
 }
 
@@ -71,7 +71,7 @@ sub new {
 sub to_string {
   my ($self) = @_;
   
-  return $self->class_name;
+  return $self->module_name;
 }
 
 1;
@@ -87,21 +87,21 @@ The SPVM::Builder::Resource class has methods to manipulate a resource.
 =head1 Usage
 
   my $resource = SPVM::Builder::Resource->new(
-    class_name => 'Resource::Zlib',
+    module_name => 'Resource::Zlib',
     mode => 'high_performance',
     argv => ['foo', 'bar'],
   );
 
 =head1 Fields
 
-=head2 class_name
+=head2 module_name
 
-  my $class_name = $resource->class_name;
-  $resource->class_name($class_name);
+  my $module_name = $resource->module_name;
+  $resource->module_name($module_name);
 
-Gets and sets the C<class_name> field.
+Gets and sets the C<module_name> field.
 
-This field is a class name of a resource.
+This field is a module name of a resource.
 
 =head2 mode
 
@@ -144,7 +144,7 @@ If a field is not defined, the field is set to the following default value.
 
 =over 2
 
-=item * L</"class_name">
+=item * L</"module_name">
 
 undef
 
@@ -168,7 +168,7 @@ undef
 
   my $string = $resource->to_string;
 
-Returns the L</"class_name"> field.
+Returns the L</"module_name"> field.
 
 =head1 Operators
 
@@ -182,7 +182,7 @@ Always true.
 
 =head2 stringify
 
-  my $class_name = "$resource";
+  my $module_name = "$resource";
 
 The alias for the L</"to_string"> method.
 
