@@ -45,7 +45,7 @@ struct spvm_basic_type {
   SPVM_METHOD* init_method;
   const char* parent_name;
   SPVM_BASIC_TYPE* parent;
-  int32_t fields_size;
+  int32_t fields_width;
   int8_t has_init_block;
   int8_t is_anon;
   int8_t is_precompile;
@@ -85,7 +85,7 @@ int32_t SPVM_BASIC_TYPE_is_integer_type_within_int(SPVM_COMPILER* compiler, int3
 
 int32_t SPVM_BASIC_TYPE_has_interface(SPVM_COMPILER* compiler, int32_t basic_type_id, int32_t interface_basic_type_id);
 
-int32_t SPVM_BASIC_TYPE_is_super(SPVM_COMPILER* compiler, int32_t super_basic_type_id, int32_t child_basic_type_id);
+int32_t SPVM_BASIC_TYPE_is_super_class(SPVM_COMPILER* compiler, int32_t super_basic_type_id, int32_t child_basic_type_id);
 
 SPVM_STRING* SPVM_BASIC_TYPE_add_constant_string(SPVM_COMPILER* compiler, SPVM_BASIC_TYPE* basic_type, const char* value, int32_t length);
 
