@@ -101,17 +101,13 @@ struct spvm_compiler {
   // Fail load class symtable
   SPVM_HASH* if_require_not_found_module_name_symtable;
   
+  SPVM_LIST* cur_anon_op_types;
+  
   // Basic types
   SPVM_LIST* basic_types;
   
   // Basic type symbol table
   SPVM_HASH* basic_type_symtable;
-  
-  SPVM_LIST* cur_anon_op_types;
-  
-  // SPVM runtime codes
-  int32_t* runtime_codes;
-  
 };
 
 SPVM_COMPILER* SPVM_COMPILER_new();
@@ -141,16 +137,6 @@ const char* SPVM_COMPILER_get_error_message(SPVM_COMPILER* compiler, int32_t ind
 
 int32_t SPVM_COMPILER_compile(SPVM_COMPILER* compiler, const char* basic_type_name);
 
-int32_t SPVM_COMPILER_calculate_runtime_codes_length(SPVM_COMPILER* compiler);
-int32_t* SPVM_COMPILER_create_runtime_codes(SPVM_COMPILER* compiler, SPVM_ALLOCATOR* allocator);
-
-int32_t SPVM_COMPILER_get_class_vars_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_fields_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_methods_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_anon_basic_types_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_args_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_opcodes_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_constant_strings_length(SPVM_COMPILER* compiler);
-int32_t SPVM_COMPILER_get_string_pool_length(SPVM_COMPILER* compiler);
+SPVM_RUNTIME* SPVM_COMPILER_build_runtime(SPVM_COMPILER* compiler);
 
 #endif
