@@ -9,7 +9,7 @@
 #include <string.h>
 #include <stdarg.h>
 
-#define SPVM_NATIVE_VERSION_NUMBER 0.989019
+#define SPVM_NATIVE_VERSION_NUMBER 0.989020
 
 #define SPVM_NATIVE_CREATE_VERSION_STRING_STRINGIFY(x) #x
 
@@ -26,6 +26,9 @@ typedef struct spvm_env_api SPVM_ENV_API;
 struct spvm_env_compiler;
 typedef struct spvm_env_compiler SPVM_ENV_COMPILER;
 
+struct spvm_env_module_file;
+typedef struct spvm_env_module_file SPVM_ENV_MODULE_FILE;
+
 struct spvm_env_runtime;
 typedef struct spvm_env_runtime SPVM_ENV_RUNTIME;
 
@@ -262,7 +265,7 @@ struct spvm_env {
   int32_t (*has_interface)(SPVM_ENV* env, SPVM_VALUE* stack, void* object, void* interface_basic_type);
   void (*print)(SPVM_ENV* env, SPVM_VALUE* stack, void* string);
   void (*print_stderr)(SPVM_ENV* env, SPVM_VALUE* stack, void* string);
-  int32_t (*init_env)(SPVM_ENV* env);
+  void* reserved168;
   int32_t (*call_init_blocks)(SPVM_ENV* env, SPVM_VALUE* stack);
   void* reserved170;
   SPVM_VALUE* (*new_stack)(SPVM_ENV* env);
@@ -316,15 +319,16 @@ struct spvm_env {
   void* (*get_basic_type)(SPVM_ENV* env, SPVM_VALUE* stack, const char* basic_type_name);
   void* (*get_basic_type_by_name)(SPVM_ENV* env, SPVM_VALUE* stack, const char* basic_type_name, int32_t* error, const char* func_name, const char* file, int32_t line);
   void* (*get_basic_type_by_id)(SPVM_ENV* env, SPVM_VALUE* stack, int32_t basic_type_id);
+  void* (*get_object_basic_type)(SPVM_ENV* env, SPVM_VALUE* stack, void* object);
 };
 
 struct spvm_env_runtime {
   void* (*new_instance)(void);
   void (*free_instance)(void* runtime);
-  int32_t* (*get_opcodes)(void* runtime);
+  void* reserved2;
   void* reserved3;
-  int32_t* (*get_runtime_codes)(void* runtime);
-  int32_t (*get_runtime_codes_length)(void* runtime);
+  void* reserved4;
+  void* reserved5;
   void* reserved6;
   void* reserved7;
   void* reserved8;
@@ -369,18 +373,18 @@ struct spvm_env_runtime {
   int32_t (*get_method_is_anon)(void* runtime, void* method);
   int32_t (*get_method_is_native)(void* runtime, void* method);
   int32_t (*get_method_is_precompile)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_byte_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_short_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_int_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_long_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_float_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_double_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_object_vars_length)(void* runtime, void* method);
-  int32_t (*get_method_call_stack_ref_vars_length)(void* runtime, void* method);
+  int32_t (*get_method_byte_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_short_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_int_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_long_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_float_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_double_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_object_vars_width)(void* runtime, void* method);
+  int32_t (*get_method_ref_vars_width)(void* runtime, void* method);
   int32_t (*get_method_mortal_stack_length)(void* runtime, void* method);
-  int32_t (*get_method_opcodes_base)(void* runtime, void* method);
+  void* reserved59;
   int32_t (*get_method_opcodes_length)(void* runtime, void* method);
-  int32_t (*get_method_args_base)(void* runtime, void* method);
+  void* reserved61;
   int32_t (*get_method_args_length)(void* runtime, void* method);
   void* reserved63;
   void* reserved64;
@@ -395,8 +399,8 @@ struct spvm_env_runtime {
   void* reserved73;
   void* reserved74;
   void* object_length_offset;
-  void* (*get_allocator)(void* runtime);
-  void (*build)(void* runtime, int32_t* runtime_codes);
+  void* reserved76;
+  void* reserved77;
   void* reserved78;
   int32_t (*get_method_required_args_length)(void* runtime, void* method);
   void* reserved80;
@@ -409,11 +413,11 @@ struct spvm_env_runtime {
   const char* (*get_basic_type_module_rel_file)(void* runtime, void* basic_type);
   const char* (*get_basic_type_module_dir)(void* runtime, void* basic_type);
   int32_t (*get_basic_type_is_anon)(void* runtime, void* basic_type);
-  int32_t (*get_basic_type_fields_base)(void* runtime, void* basic_type);
+  void* reserved90;
   int32_t (*get_basic_type_fields_length)(void* runtime, void* basic_type);
-  int32_t (*get_basic_type_methods_base)(void* runtime, void* basic_type);
+  void* reserved92;
   int32_t (*get_basic_type_methods_length)(void* runtime, void* basic_type);
-  int32_t (*get_basic_type_class_vars_base)(void* runtime, void* basic_type);
+  void* reserved94;
   int32_t (*get_basic_type_class_vars_length)(void* runtime, void* basic_type);
   void* reserved96;
   void* reserved97;
@@ -471,12 +475,31 @@ struct spvm_env_compiler {
   int32_t (*compile)(void* compiler, const char* basic_type_name);
   int32_t (*get_error_messages_length)(void* compiler);
   const char* (*get_error_message)(void* compiler, int32_t index);
-  int32_t* (*create_runtime_codes)(void* compiler, void* allocator);
+  void* reserved12;
   void (*clear_include_dirs)(void* compiler);
+  void* (*get_module_file)(void* compiler, const char* module_name);
+  void (*add_module_file)(void* compiler, const char* module_name, void* module_file);
+  void* (*build_runtime)(void* compiler);
 };
 
-SPVM_ENV* SPVM_NATIVE_new_env_raw();
-SPVM_ENV* SPVM_NATIVE_new_env_prepared();
+struct spvm_env_module_file {
+  SPVM_ENV_MODULE_FILE* (*new_env)(void);
+  void* (*new_instance)(void* compiler);
+  const char* (*get_module_name)(void* compiler, void* module_file);
+  void (*set_module_name)(void* compiler, void* module_file, void* module_name);
+  const char* (*get_file)(void* compiler, void* module_file);
+  void (*set_file)(void* compiler, void* module_file, void* file);
+  const char* (*get_dir)(void* compiler, void* module_file);
+  void (*set_dir)(void* compiler, void* module_file, void* dir);
+  const char* (*get_rel_file)(void* compiler, void* module_file);
+  void (*set_rel_file)(void* compiler, void* module_file, void* rel_file);
+  const char* (*get_content)(void* compiler, void* module_file);
+  void (*set_content)(void* compiler, void* module_file, void* content);
+  int32_t (*get_content_length)(void* compiler, void* module_file);
+  void (*set_content_length)(void* compiler, void* module_file, int32_t content_length);
+};
+
+SPVM_ENV* SPVM_NATIVE_new_env_raw(void);
 
 struct spvm_env_allocator {
   void* (*new_instance)(void);
@@ -495,6 +518,7 @@ struct spvm_env_api {
   SPVM_ENV_STRING_BUFFER* string_buffer;
   SPVM_ENV_COMPILER* compiler;
   SPVM_ENV_RUNTIME* runtime;
+  SPVM_ENV_MODULE_FILE* module_file;
 };
 
 #define spvm_warn(format, ...) fprintf(stderr, format "\n", ##__VA_ARGS__)
