@@ -7,8 +7,8 @@
 #include "spvm_typedecl.h"
 
 struct spvm_runtime_arg {
+  SPVM_RUNTIME_BASIC_TYPE* basic_type;
   int32_t index;
-  int32_t basic_type_id;
   int32_t type_dimension;
   int32_t type_flag;
 };
