@@ -15,6 +15,8 @@
 #include "spvm_method.h"
 #include "spvm_string_buffer.h"
 #include "spvm_allocator.h"
+#include "spvm_string.h"
+#include "spvm_module_file.h"
 
 SPVM_ENV_COMPILER* SPVM_API_COMPILER_new_env() {
   void* env_compiler_init[]  = {
@@ -30,8 +32,11 @@ SPVM_ENV_COMPILER* SPVM_API_COMPILER_new_env() {
     SPVM_API_COMPILER_compile,
     SPVM_API_COMPILER_get_error_messages_length,
     SPVM_API_COMPILER_get_error_message,
-    SPVM_API_COMPILER_create_runtime_codes,
+    NULL, // reserved12
     SPVM_API_COMPILER_clear_include_dirs,
+    SPVM_API_COMPILER_get_module_file,
+    SPVM_API_COMPILER_add_module_file,
+    SPVM_API_COMPILER_build_runtime,
   };
   SPVM_ENV_COMPILER* env_compiler = calloc(1, sizeof(env_compiler_init));
   memcpy(env_compiler, env_compiler_init, sizeof(env_compiler_init));
@@ -60,7 +65,13 @@ int32_t SPVM_API_COMPILER_get_start_line(SPVM_COMPILER* compiler) {
 }
 
 void SPVM_API_COMPILER_set_start_file(SPVM_COMPILER* compiler, const char* start_file) {
-  SPVM_COMPILER_set_start_file(compiler, start_file);
+  if (start_file) {
+    SPVM_STRING* start_file_string = SPVM_STRING_new(compiler, start_file, strlen(start_file));
+    SPVM_COMPILER_set_start_file(compiler, start_file_string->value);
+  }
+  else {
+    SPVM_COMPILER_set_start_file(compiler, NULL);
+  }
 }
 
 const char* SPVM_API_COMPILER_get_start_file(SPVM_COMPILER* compiler) {
@@ -68,8 +79,13 @@ const char* SPVM_API_COMPILER_get_start_file(SPVM_COMPILER* compiler) {
   return start_file;
 }
 
-void SPVM_API_COMPILER_add_include_dir(SPVM_COMPILER* compiler, const char* include_dir) {  
-  SPVM_COMPILER_add_include_dir(compiler, include_dir);
+void SPVM_API_COMPILER_add_include_dir(SPVM_COMPILER* compiler, const char* include_dir) {
+  assert(include_dir);
+  
+  if (include_dir) {
+    SPVM_STRING* include_dir_string = SPVM_STRING_new(compiler, include_dir, strlen(include_dir));
+    SPVM_COMPILER_add_include_dir(compiler, include_dir_string->value);
+  }
 }
 
 int32_t SPVM_API_COMPILER_get_include_dirs_length (SPVM_COMPILER* compiler) {
@@ -100,9 +116,26 @@ const char* SPVM_API_COMPILER_get_error_message(SPVM_COMPILER* compiler, int32_t
   return SPVM_COMPILER_get_error_message(compiler, index);
 }
 
-int32_t* SPVM_API_COMPILER_create_runtime_codes(SPVM_COMPILER* compiler, SPVM_ALLOCATOR* allocator) {
+SPVM_RUNTIME* SPVM_API_COMPILER_build_runtime(SPVM_COMPILER* compiler) {
   
-  int32_t* runtime_codes = SPVM_COMPILER_create_runtime_codes(compiler, allocator);
+  SPVM_RUNTIME* runtime = SPVM_COMPILER_build_runtime(compiler);
   
-  return runtime_codes;
+  return runtime;
+}
+
+SPVM_MODULE_FILE* SPVM_API_COMPILER_get_module_file(SPVM_COMPILER* compiler, const char* module_name) {
+  SPVM_MODULE_FILE* module_file = SPVM_COMPILER_get_module_file(compiler, module_name);
+  
+  return module_file;
+}
+
+void SPVM_API_COMPILER_add_module_file(SPVM_COMPILER* compiler, const char* module_name, SPVM_MODULE_FILE* module_file) {
+  if (module_name) {
+    SPVM_STRING* module_name_string = SPVM_STRING_new(compiler, module_name, strlen(module_name));
+    module_name = module_name_string->value;
+  }
+  
+  SPVM_COMPILER_add_module_file(compiler, module_name, module_file);
+  
+  return;
 }
