@@ -99,10 +99,10 @@ SPVM_ENV_RUNTIME* SPVM_API_RUNTIME_new_env() {
   void* env_runtime_init[]  = {
     SPVM_API_RUNTIME_new_instance,
     SPVM_API_RUNTIME_free_instance,
-    SPVM_API_RUNTIME_get_opcodes,
+    NULL, // reserved2
     NULL, // reserved3
-    SPVM_API_RUNTIME_get_runtime_codes,
-    SPVM_API_RUNTIME_get_runtime_codes_length,
+    NULL, // reserved4
+    NULL, // reserved5,
     NULL, // reserved6
     NULL, // reserved7,
     NULL, // reserved8,
@@ -147,18 +147,18 @@ SPVM_ENV_RUNTIME* SPVM_API_RUNTIME_new_env() {
     SPVM_API_RUNTIME_get_method_is_anon,
     SPVM_API_RUNTIME_get_method_is_native,
     SPVM_API_RUNTIME_get_method_is_precompile,
-    SPVM_API_RUNTIME_get_method_call_stack_byte_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_short_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_int_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_long_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_float_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_double_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_object_vars_length,
-    SPVM_API_RUNTIME_get_method_call_stack_ref_vars_length,
+    SPVM_API_RUNTIME_get_method_byte_vars_width,
+    SPVM_API_RUNTIME_get_method_short_vars_width,
+    SPVM_API_RUNTIME_get_method_int_vars_width,
+    SPVM_API_RUNTIME_get_method_long_vars_width,
+    SPVM_API_RUNTIME_get_method_float_vars_width,
+    SPVM_API_RUNTIME_get_method_double_vars_width,
+    SPVM_API_RUNTIME_get_method_object_vars_width,
+    SPVM_API_RUNTIME_get_method_ref_vars_width,
     SPVM_API_RUNTIME_get_method_mortal_stack_length,
-    SPVM_API_RUNTIME_get_method_opcodes_base,
+    NULL, // reserved59
     SPVM_API_RUNTIME_get_method_opcodes_length,
-    SPVM_API_RUNTIME_get_method_args_base,
+    NULL, // reserved 61
     SPVM_API_RUNTIME_get_method_args_length,
     NULL, // reserved63
     NULL, // reserved64
@@ -173,8 +173,8 @@ SPVM_ENV_RUNTIME* SPVM_API_RUNTIME_new_env() {
     NULL, // reserved73
     NULL, // reserved74
     (void*)(intptr_t)offsetof(SPVM_OBJECT, length), // object_length_offset
-    SPVM_API_RUNTIME_get_allocator,
-    SPVM_API_RUNTIME_build,
+    NULL, // reserved76
+    NULL, // reserved77
     NULL, // reserved78
     SPVM_API_RUNTIME_get_method_required_args_length,
     NULL, // reserved80
@@ -187,11 +187,11 @@ SPVM_ENV_RUNTIME* SPVM_API_RUNTIME_new_env() {
     SPVM_API_RUNTIME_get_basic_type_module_rel_file,
     SPVM_API_RUNTIME_get_basic_type_module_dir,
     SPVM_API_RUNTIME_get_basic_type_is_anon,
-    SPVM_API_RUNTIME_get_basic_type_fields_base,
+    NULL, // reserved90
     SPVM_API_RUNTIME_get_basic_type_fields_length,
-    SPVM_API_RUNTIME_get_basic_type_methods_base,
+    NULL, // reserved92
     SPVM_API_RUNTIME_get_basic_type_methods_length,
-    SPVM_API_RUNTIME_get_basic_type_class_vars_base,
+    NULL, // reserved94
     SPVM_API_RUNTIME_get_basic_type_class_vars_length,
     NULL, // reserved96
     NULL, // reserved97
@@ -248,32 +248,17 @@ SPVM_RUNTIME* SPVM_API_RUNTIME_new_instance() {
 }
 
 void SPVM_API_RUNTIME_free_instance(SPVM_RUNTIME* runtime) {
-
+  
   SPVM_RUNTIME_free(runtime);
 }
 
-SPVM_OPCODE* SPVM_API_RUNTIME_get_opcodes(SPVM_RUNTIME* runtime) {
-
-  return runtime->opcodes;
-}
-
 int32_t SPVM_API_RUNTIME_get_basic_types_length(SPVM_RUNTIME* runtime) {
   
   return runtime->basic_types_length;
 }
 
-int32_t* SPVM_API_RUNTIME_get_runtime_codes(SPVM_RUNTIME* runtime) {
-
-  return runtime->runtime_codes;
-}
-
-int32_t SPVM_API_RUNTIME_get_runtime_codes_length(SPVM_RUNTIME* runtime) {
-  
-  return runtime->runtime_codes_length;
-}
-
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_basic_type_by_id(SPVM_RUNTIME* runtime, int32_t basic_type_id) {
-
+  
   if (basic_type_id < 0) {
     return NULL;
   }
@@ -295,164 +280,83 @@ SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_basic_type_by_name(SPVM_RUNTIME* r
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_id(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
-  int32_t basic_type_id = basic_type->id;
   
-  return basic_type_id;
-}
-
-int32_t SPVM_API_RUNTIME_get_basic_type_anon_basic_type_id(SPVM_RUNTIME* runtime, int32_t basic_type_id, int32_t anon_basic_type_index) {
-  
-  SPVM_RUNTIME_BASIC_TYPE* basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, basic_type_id);
-  
-  int32_t anon_basic_type_id = runtime->anon_basic_type_ids[basic_type->anon_basic_types_base + anon_basic_type_index];
-  
-  return anon_basic_type_id;
-}
-
-const char* SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t constant_string_index) {
-  
-  const char* constant_string_value = SPVM_API_RUNTIME_get_basic_type_constant_string_value(runtime, basic_type, constant_string_index, NULL);
-  
-  return constant_string_value;
-}
-
-const char* SPVM_API_RUNTIME_get_basic_type_constant_string_value(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t constant_string_index, int32_t* string_length) {
-  if (constant_string_index < 0) {
-    return NULL;
-  }
-  
-  SPVM_RUNTIME_STRING* constant_string = &runtime->constant_strings[basic_type->constant_strings_base + constant_string_index];
-  
-  const char* constant_string_value = constant_string->value;
-  
-  if (string_length) {
-    *string_length = constant_string->length;
-  }
-  
-  return constant_string_value;
+  return basic_type->id;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_category(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t basic_type_category = basic_type->category;
-  
-  return basic_type_category;
+  return basic_type->category;
 }
 
 const char* SPVM_API_RUNTIME_get_basic_type_name(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  const char* basic_type_name = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, basic_type, basic_type->name_string_index);
-  
-  return basic_type_name;
+  return  basic_type->name;
 }
 
 const char* SPVM_API_RUNTIME_get_basic_type_version_string(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  const char* version_string = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, basic_type, basic_type->version_string_string_index);
-  
-  return version_string;
+  return basic_type->version_string;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_is_pointer(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t is_pointer = basic_type->is_pointer;
-  
-  return is_pointer;
-}
-
-int32_t SPVM_API_RUNTIME_get_basic_type_anon_basic_types_base(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
-  
-  int32_t anon_basic_types_base = basic_type->anon_basic_types_base;
-  
-  return anon_basic_types_base;
+  return basic_type->is_pointer;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_anon_basic_types_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t anon_basic_types_length = basic_type->anon_basic_types_length;
-  
-  return anon_basic_types_length;
+  return basic_type->anon_basic_types_length;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_basic_type_anon_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t anon_basic_type_index) {
   
-  int32_t anon_basic_type_id = runtime->anon_basic_type_ids[basic_type->anon_basic_types_base + anon_basic_type_index];
+  if (anon_basic_type_index < 0) {
+    return NULL;
+  }
   
-  SPVM_RUNTIME_BASIC_TYPE* anon_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, anon_basic_type_id);
+  if (anon_basic_type_index >= basic_type->anon_basic_types_length) {
+    return NULL;
+  }
+  
+  SPVM_RUNTIME_BASIC_TYPE* anon_basic_type = &basic_type->anon_basic_types[anon_basic_type_index];
   
   return anon_basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_is_anon(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t is_anon = basic_type->is_anon;
-  
-  return is_anon;
+  return basic_type->is_anon;
 }
 
 const char* SPVM_API_RUNTIME_get_basic_type_module_rel_file(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  const char* module_rel_file = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, basic_type, basic_type->module_rel_file_string_index);
-  
-  return module_rel_file;
+  return basic_type->module_rel_file;
 }
 
 const char* SPVM_API_RUNTIME_get_basic_type_module_dir(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  const char* module_dir = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, basic_type, basic_type->module_dir_string_index);
-  
-  return module_dir;
-}
-
-int32_t SPVM_API_RUNTIME_get_basic_type_fields_base(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
-  
-  int32_t fields_base = basic_type->fields_base;
-  
-  return fields_base;
+  return basic_type->module_dir;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_fields_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t fields_length = basic_type->fields_length;
-  
-  return fields_length;
-}
-
-int32_t SPVM_API_RUNTIME_get_basic_type_methods_base(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
-  
-  int32_t methods_base = basic_type->methods_base;
-  
-  return methods_base;
+  return basic_type->fields_length;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_methods_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t methods_length = basic_type->methods_length;
-  
-  return methods_length;
-}
-
-int32_t SPVM_API_RUNTIME_get_basic_type_class_vars_base(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
-  
-  int32_t class_vars_base = basic_type->class_vars_base;
-  
-  return class_vars_base;
+  return basic_type->methods_length;
 }
 
 int32_t SPVM_API_RUNTIME_get_basic_type_class_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t class_vars_length = basic_type->class_vars_length;
-  
-  return class_vars_length;
+  return basic_type->class_vars_length;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_basic_type_parent(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type) {
   
-  int32_t parent_basic_type_id = basic_type->parent_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* parent_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, parent_basic_type_id);
-  
-  return parent_basic_type;
+  return basic_type->parent;
 }
 
 SPVM_RUNTIME_CLASS_VAR* SPVM_API_RUNTIME_get_class_var(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t class_var_index) {
@@ -465,7 +369,7 @@ SPVM_RUNTIME_CLASS_VAR* SPVM_API_RUNTIME_get_class_var(SPVM_RUNTIME* runtime, SP
     return NULL;
   }
   
-  SPVM_RUNTIME_CLASS_VAR* class_var = &runtime->class_vars[basic_type->class_vars_base + class_var_index];
+  SPVM_RUNTIME_CLASS_VAR* class_var = &basic_type->class_vars[class_var_index];
   
   return class_var;
 }
@@ -489,48 +393,32 @@ SPVM_RUNTIME_CLASS_VAR* SPVM_API_RUNTIME_get_class_var_by_name(SPVM_RUNTIME* run
 
 int32_t SPVM_API_RUNTIME_get_class_var_index(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* class_var) {
   
-  int32_t index = class_var->index;
-  
-  return index;
+  return class_var->index;
 }
 
 const char* SPVM_API_RUNTIME_get_class_var_name(SPVM_RUNTIME* runtime, SPVM_RUNTIME_CLASS_VAR* class_var) {
   
-  const char* class_var_name = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, SPVM_API_RUNTIME_get_class_var_current_basic_type(runtime, class_var), class_var->name_string_index);
-  
-  return class_var_name;
+  return class_var->name;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_class_var_current_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_CLASS_VAR* class_var) {
   
-  int32_t current_basic_type_id = class_var->current_basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* current_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, current_basic_type_id);
-  
-  return current_basic_type;
+  return class_var->current_basic_type;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_class_var_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_CLASS_VAR* class_var) {
   
-  int32_t basic_type_id = class_var->basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, basic_type_id);
-  
-  return basic_type;
+  return class_var->basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_get_class_var_type_dimension(SPVM_RUNTIME* runtime, SPVM_RUNTIME_CLASS_VAR* class_var) {
   
-  int32_t type_dimension = class_var->type_dimension;
-  
-  return type_dimension;
+  return class_var->type_dimension;
 }
 
 int32_t SPVM_API_RUNTIME_get_class_var_type_flag(SPVM_RUNTIME* runtime, SPVM_RUNTIME_CLASS_VAR* class_var) {
   
-  int32_t type_flag = class_var->type_flag;
-  
-  return type_flag;
+  return class_var->type_flag;
 }
 
 SPVM_RUNTIME_FIELD* SPVM_API_RUNTIME_get_field(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t field_index) {
@@ -543,7 +431,7 @@ SPVM_RUNTIME_FIELD* SPVM_API_RUNTIME_get_field(SPVM_RUNTIME* runtime, SPVM_RUNTI
     return NULL;
   }
   
-  SPVM_RUNTIME_FIELD* field = &runtime->fields[basic_type->fields_base + field_index];
+  SPVM_RUNTIME_FIELD* field = &basic_type->fields[field_index];
   
   return field;
 }
@@ -567,55 +455,37 @@ SPVM_RUNTIME_FIELD* SPVM_API_RUNTIME_get_field_by_name(SPVM_RUNTIME* runtime, SP
 
 int32_t SPVM_API_RUNTIME_get_field_index(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  int32_t index = field->index;
-  
-  return index;
+  return field->index;
 }
 
 int32_t SPVM_API_RUNTIME_get_field_offset(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  int32_t offset = field->offset;
-  
-  return offset;
+  return field->offset;
 }
 
 const char* SPVM_API_RUNTIME_get_field_name(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  const char* field_name = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, SPVM_API_RUNTIME_get_field_current_basic_type(runtime, field), field->name_string_index);
-  
-  return field_name;
+  return field->name;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_field_current_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  int32_t current_basic_type_id = field->current_basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* current_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, current_basic_type_id);
-  
-  return current_basic_type;
+  return field->current_basic_type;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_field_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  int32_t basic_type_id = field->basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, basic_type_id);
-  
-  return basic_type;
+  return field->basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_get_field_type_dimension(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  int32_t type_dimension = field->type_dimension;
-  
-  return type_dimension;
+  return field->type_dimension;
 }
 
 int32_t SPVM_API_RUNTIME_get_field_type_flag(SPVM_RUNTIME* runtime, SPVM_RUNTIME_FIELD* field) {
   
-  int32_t type_flag = field->type_flag;
-  
-  return type_flag;
+  return  field->type_flag;
 }
 
 SPVM_RUNTIME_METHOD* SPVM_API_RUNTIME_get_method(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t method_index) {
@@ -628,7 +498,7 @@ SPVM_RUNTIME_METHOD* SPVM_API_RUNTIME_get_method(SPVM_RUNTIME* runtime, SPVM_RUN
     return NULL;
   }
   
-  SPVM_RUNTIME_METHOD* method = &runtime->methods[basic_type->methods_base + method_index];
+  SPVM_RUNTIME_METHOD* method = &basic_type->methods[method_index];
   
   return method;
 }
@@ -649,7 +519,7 @@ SPVM_RUNTIME_METHOD* SPVM_API_RUNTIME_get_method_by_name(SPVM_RUNTIME* runtime,
       int32_t cur_half_index = cur_min_index +(cur_max_index - cur_min_index) / 2;
       
       SPVM_RUNTIME_METHOD* method = SPVM_API_RUNTIME_get_method(runtime, basic_type, cur_half_index);
-      const char* cur_half_method_name = SPVM_API_RUNTIME_get_method_name(runtime, method);
+      const char* cur_half_method_name = method->name;
       
       int32_t cmp_result = strcmp(method_name, cur_half_method_name);
       
@@ -671,46 +541,28 @@ SPVM_RUNTIME_METHOD* SPVM_API_RUNTIME_get_method_by_name(SPVM_RUNTIME* runtime,
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_method_current_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t current_basic_type_id = method->current_basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* current_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, current_basic_type_id);
-  
-  return current_basic_type;
-}
-
-int32_t SPVM_API_RUNTIME_get_method_opcodes_base(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
-  
-  int32_t opcodes_base = method->opcodes_base;
-  
-  return opcodes_base;
+  return method->current_basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_opcodes_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t opcodes_length = method->opcodes_length;
   
-  return opcodes_length;
+  return method->opcodes_length;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_index(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t index = method->index;
-  
-  return index;
+  return method->index;
 }
 
 const char* SPVM_API_RUNTIME_get_method_name(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  const char* method_name = SPVM_API_RUNTIME_get_basic_type_constant_string_value_nolen(runtime, SPVM_API_RUNTIME_get_method_current_basic_type(runtime, method), method->name_string_index);
-  
-  return method_name;
+  return method->name;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_is_anon(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t is_anon = method->is_anon;
-  
-  return is_anon;
+  return method->is_anon;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_is_native(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
@@ -730,116 +582,79 @@ int32_t SPVM_API_RUNTIME_get_method_is_enum(SPVM_RUNTIME* runtime, SPVM_RUNTIME_
 
 int32_t SPVM_API_RUNTIME_get_method_args_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t args_length = method->args_length;
-  
-  return args_length;
+  return method->args_length;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_required_args_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t required_args_length = method->required_args_length;
-  
-  return required_args_length;
-}
-
-int32_t SPVM_API_RUNTIME_get_method_args_base(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
-  
-  int32_t args_base = method->args_base;
-  
-  return args_base;
+  return method->required_args_length;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_is_class_method(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t is_class_method = method->is_class_method;
-  
-  return is_class_method;
+  return method->is_class_method;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_method_return_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t return_basic_type_id = method->return_basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* return_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, return_basic_type_id);
-  
-  return return_basic_type;
+  return method->return_basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_return_type_dimension(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t return_type_dimension = method->return_type_dimension;
-  
-  return return_type_dimension;
+  return method->return_type_dimension;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_return_type_flag(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t return_type_flag = method->return_type_flag;
-  
-  return return_type_flag;
+  return method->return_type_flag;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_byte_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_byte_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_byte_vars_length = method->call_stack_byte_vars_length;
+  int32_t byte_vars_width = method->byte_vars_width;
   
-  return call_stack_byte_vars_length;
+  return byte_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_short_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_short_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_short_vars_length = method->call_stack_short_vars_length;
-  
-  return call_stack_short_vars_length;
+  return method->short_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_int_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_int_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_int_vars_length = method->call_stack_int_vars_length;
-  
-  return call_stack_int_vars_length;
+  return method->int_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_long_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_long_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_long_vars_length = method->call_stack_long_vars_length;
-  
-  return call_stack_long_vars_length;
+  return method->long_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_float_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_float_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_float_vars_length = method->call_stack_float_vars_length;
-  
-  return call_stack_float_vars_length;
+  return method->float_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_double_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_double_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_double_vars_length = method->call_stack_double_vars_length;
-  
-  return call_stack_double_vars_length;
+  return method->double_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_object_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_object_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_object_vars_length = method->call_stack_object_vars_length;
-  
-  return call_stack_object_vars_length;
+  return method->object_vars_width;
 }
 
-int32_t SPVM_API_RUNTIME_get_method_call_stack_ref_vars_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
+int32_t SPVM_API_RUNTIME_get_method_ref_vars_width(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t call_stack_ref_vars_length = method->call_stack_ref_vars_length;
-  
-  return call_stack_ref_vars_length;
+  return method->ref_vars_width;
 }
 
 int32_t SPVM_API_RUNTIME_get_method_mortal_stack_length(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
   
-  int32_t mortal_stack_length = method->mortal_stack_length;
-  
-  return mortal_stack_length;
+  return method->mortal_stack_length;
 }
 
 SPVM_RUNTIME_ARG* SPVM_API_RUNTIME_get_arg(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method, int32_t arg_index) {
@@ -852,32 +667,24 @@ SPVM_RUNTIME_ARG* SPVM_API_RUNTIME_get_arg(SPVM_RUNTIME* runtime, SPVM_RUNTIME_M
     return NULL;
   }
   
-  SPVM_RUNTIME_ARG* arg = &runtime->args[method->args_base + arg_index];
+  SPVM_RUNTIME_ARG* arg = &method->args[arg_index];
   
   return arg;
 }
 
 SPVM_RUNTIME_BASIC_TYPE* SPVM_API_RUNTIME_get_arg_basic_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_ARG* arg) {
   
-  int32_t basic_type_id = arg->basic_type_id;
-  
-  SPVM_RUNTIME_BASIC_TYPE* basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, basic_type_id);
-  
-  return basic_type;
+  return arg->basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_get_arg_type_dimension(SPVM_RUNTIME* runtime, SPVM_RUNTIME_ARG* arg) {
   
-  int32_t type_dimension = arg->type_dimension;
-  
-  return type_dimension;
+  return arg->type_dimension;
 }
 
 int32_t SPVM_API_RUNTIME_get_arg_type_flag(SPVM_RUNTIME* runtime, SPVM_RUNTIME_ARG* arg) {
   
-  int32_t type_flag = arg->type_flag;
-  
-  return type_flag;
+  return arg->type_flag;
 }
 
 void* SPVM_API_RUNTIME_get_native_method_address(SPVM_RUNTIME* runtime, SPVM_RUNTIME_METHOD* method) {
@@ -900,32 +707,6 @@ void SPVM_API_RUNTIME_set_precompile_method_address(SPVM_RUNTIME* runtime, SPVM_
   method->precompile_address = address;
 }
 
-int32_t SPVM_API_RUNTIME_has_interface_by_id(SPVM_RUNTIME* runtime, int32_t basic_type_id, int32_t interface_basic_type_id) {
-
-  SPVM_RUNTIME_BASIC_TYPE* basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, basic_type_id);
-  SPVM_RUNTIME_BASIC_TYPE* interface_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, interface_basic_type_id);
-  
-  if (!(basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_CLASS || basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_INTERFACE)) {
-    return 0;
-  }
-  
-  if (!(interface_basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_CLASS || interface_basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_INTERFACE)) {
-    return 0;
-  }
-  
-  SPVM_RUNTIME_METHOD* method_interface = SPVM_API_RUNTIME_get_method(runtime, interface_basic_type, interface_basic_type->required_method_index);
-  
-  const char* method_interface_name =  SPVM_API_RUNTIME_get_basic_type_constant_string_value(runtime, SPVM_API_RUNTIME_get_method_current_basic_type(runtime, method_interface), method_interface->name_string_index, NULL);
-  
-  SPVM_RUNTIME_METHOD* found_method = SPVM_API_RUNTIME_get_method_by_name(runtime, basic_type, method_interface_name);
-  if (found_method) {
-    return 1;
-  }
-  else {
-    return 0;
-  }
-}
-
 int32_t SPVM_API_RUNTIME_has_interface(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, SPVM_RUNTIME_BASIC_TYPE* interface_basic_type) {
 
   if (!(basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_CLASS || basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_INTERFACE)) {
@@ -936,9 +717,9 @@ int32_t SPVM_API_RUNTIME_has_interface(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC
     return 0;
   }
   
-  SPVM_RUNTIME_METHOD* method_interface = SPVM_API_RUNTIME_get_method(runtime, interface_basic_type, interface_basic_type->required_method_index);
+  SPVM_RUNTIME_METHOD* method_interface = interface_basic_type->required_method;
   
-  const char* method_interface_name =  SPVM_API_RUNTIME_get_basic_type_constant_string_value(runtime, SPVM_API_RUNTIME_get_method_current_basic_type(runtime, method_interface), method_interface->name_string_index, NULL);
+  const char* method_interface_name = method_interface->name;
   
   SPVM_RUNTIME_METHOD* found_method = SPVM_API_RUNTIME_get_method_by_name(runtime, basic_type, method_interface_name);
   if (found_method) {
@@ -949,9 +730,9 @@ int32_t SPVM_API_RUNTIME_has_interface(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC
   }
 }
 
-int32_t SPVM_API_RUNTIME_is_super(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* super_basic_type, SPVM_RUNTIME_BASIC_TYPE* child_basic_type) {
+int32_t SPVM_API_RUNTIME_is_super_class(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* super_basic_type, SPVM_RUNTIME_BASIC_TYPE* child_basic_type) {
 
-  int32_t is_super_basic_type = 0;
+  int32_t is_super_class_basic_type = 0;
   
   if (!(super_basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_CLASS)) {
     return 0;
@@ -961,40 +742,32 @@ int32_t SPVM_API_RUNTIME_is_super(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE
     return 0;
   }
   
-  int32_t parent_basic_type_id = child_basic_type->parent_id;
+  SPVM_RUNTIME_BASIC_TYPE* parent_basic_type = child_basic_type->parent;
+  
   while (1) {
-    if (parent_basic_type_id > 0) {
-      SPVM_RUNTIME_BASIC_TYPE* parent_basic_type = SPVM_API_RUNTIME_get_basic_type_by_id(runtime, parent_basic_type_id);
+    if (parent_basic_type) {
       if (parent_basic_type->id == super_basic_type->id) {
-        is_super_basic_type = 1;
+        is_super_class_basic_type = 1;
         break;
       }
       else {
-        parent_basic_type_id = parent_basic_type->parent_id;
+        parent_basic_type = parent_basic_type->parent;
       }
     }
     else {
-      is_super_basic_type = 0;
+      is_super_class_basic_type = 0;
       break;
     }
   }
   
-  return is_super_basic_type;
-}
-
-SPVM_ALLOCATOR* SPVM_API_RUNTIME_get_allocator(SPVM_RUNTIME* runtime) {
-  return SPVM_RUNTIME_get_allocator(runtime);
-}
-
-void SPVM_API_RUNTIME_build(SPVM_RUNTIME* runtime, int32_t* runtime_codes) {
-  SPVM_RUNTIME_build(runtime, runtime_codes);
+  return is_super_class_basic_type;
 }
 
 int32_t SPVM_API_RUNTIME_is_object_type(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TYPE* basic_type, int32_t type_dimension, int32_t flag) {
   
   int32_t is_object_type;
   if (type_dimension == 0) {
-    int32_t basic_type_category = SPVM_API_RUNTIME_get_basic_type_category(runtime, basic_type);
+    int32_t basic_type_category = basic_type->category;
     
     switch (basic_type_category) {
       case SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_STRING:
@@ -1024,8 +797,8 @@ int32_t SPVM_API_RUNTIME_can_assign(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TY
   
   int32_t isa = 0;
   
-  int32_t dist_basic_type_category = SPVM_API_RUNTIME_get_basic_type_category(runtime, dist_basic_type);
-  int32_t src_basic_type_category = SPVM_API_RUNTIME_get_basic_type_category(runtime, src_basic_type);
+  int32_t dist_basic_type_category = dist_basic_type->category;
+  int32_t src_basic_type_category = src_basic_type->category;
   
   if (dist_basic_type->id == src_basic_type->id && dist_type_dimension == src_type_dimension) {
     isa = 1;
@@ -1047,7 +820,7 @@ int32_t SPVM_API_RUNTIME_can_assign(SPVM_RUNTIME* runtime, SPVM_RUNTIME_BASIC_TY
       isa = SPVM_API_RUNTIME_has_interface(runtime, src_basic_type, dist_basic_type);
     }
     else if (dist_basic_type_category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_CLASS) {
-      isa = SPVM_API_RUNTIME_is_super(runtime, dist_basic_type, src_basic_type);
+      isa = SPVM_API_RUNTIME_is_super_class(runtime, dist_basic_type, src_basic_type);
     }
     else {
       isa = 0;
