@@ -12,9 +12,9 @@
 
 SPVM_ENV_STRING_BUFFER* SPVM_API_STRING_BUFFER_new_env() {
   void* env_string_buffer_init[]  = {
-    SPVM_API_STRING_BUFFER_new_object,
-    SPVM_API_STRING_BUFFER_free_object,
-    SPVM_API_STRING_BUFFER_get_value,
+    SPVM_API_STRING_BUFFER_new_instance,
+    SPVM_API_STRING_BUFFER_free_instance,
+    SPVM_API_STRING_BUFFER_get_string,
     SPVM_API_STRING_BUFFER_get_length,
   };
   SPVM_ENV_STRING_BUFFER* env_string_buffer = calloc(1, sizeof(env_string_buffer_init));
@@ -23,20 +23,20 @@ SPVM_ENV_STRING_BUFFER* SPVM_API_STRING_BUFFER_new_env() {
   return env_string_buffer;
 }
 
-SPVM_STRING_BUFFER* SPVM_API_STRING_BUFFER_new_object(SPVM_ALLOCATOR* allocator, int32_t capacity) {
+SPVM_STRING_BUFFER* SPVM_API_STRING_BUFFER_new_instance(SPVM_ALLOCATOR* allocator, int32_t capacity) {
   SPVM_STRING_BUFFER* string_buffer = SPVM_STRING_BUFFER_new_tmp(allocator, capacity);
   
   return string_buffer;
 }
 
-const char* SPVM_API_STRING_BUFFER_get_value(SPVM_STRING_BUFFER* string_buffer) {
-  return string_buffer->value;
+const char* SPVM_API_STRING_BUFFER_get_string(SPVM_STRING_BUFFER* string_buffer) {
+  return string_buffer->string;
 }
 
 int32_t SPVM_API_STRING_BUFFER_get_length(SPVM_STRING_BUFFER* string_buffer) {
   return string_buffer->length;
 }
 
-void SPVM_API_STRING_BUFFER_free_object(SPVM_STRING_BUFFER* string_buffer) {
+void SPVM_API_STRING_BUFFER_free_instance(SPVM_STRING_BUFFER* string_buffer) {
   SPVM_STRING_BUFFER_free(string_buffer);
 }
