@@ -38,6 +38,7 @@
 #include "spvm_string.h"
 #include "spvm_module_file.h"
 
+#include "spvm_runtime.h"
 #include "spvm_runtime_basic_type.h"
 #include "spvm_runtime_class_var.h"
 #include "spvm_runtime_field.h"
@@ -76,7 +77,11 @@ SPVM_MODULE_FILE* SPVM_COMPILER_get_module_file(SPVM_COMPILER* compiler, const c
 
 void SPVM_COMPILER_add_module_file(SPVM_COMPILER* compiler, const char* module_name, SPVM_MODULE_FILE* module_file) {
   
-  SPVM_HASH_set(compiler->module_file_symtable, module_name, strlen(module_name), (void*)module_file);
+  SPVM_MODULE_FILE* found_module_file = SPVM_COMPILER_get_module_file(compiler, module_name);
+  
+  if (!found_module_file) {
+    SPVM_HASH_set(compiler->module_file_symtable, module_name, strlen(module_name), (void*)module_file);
+  }
 }
 
 void SPVM_COMPILER_add_basic_type_core(SPVM_COMPILER* compiler, int32_t basic_type_id, int32_t basic_type_category) {
@@ -517,419 +522,206 @@ int32_t SPVM_COMPILER_compile(SPVM_COMPILER* compiler, const char* basic_type_na
   return die_error_id;
 }
 
-int32_t SPVM_COMPILER_calculate_runtime_codes_length(SPVM_COMPILER* compiler) {
+SPVM_RUNTIME* SPVM_COMPILER_build_runtime(SPVM_COMPILER* compiler) {
   
-  int32_t length = 0;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_new();
   
-  // runtime_codes_length
-  length += 1;
-  
-  // basic_types_length
-  length++;
-  
-  // basic_types
-  length += (sizeof(SPVM_RUNTIME_BASIC_TYPE) / sizeof(int32_t)) * (compiler->basic_types->length);
-  
-  // string_pool_length
-  length++;
-  
-  // string_pool
-  length += (SPVM_COMPILER_get_string_pool_length(compiler) / sizeof(int32_t));
-  
-  // constant_strings_length
-  length++;
-  
-  // constant_strings
-  length += (sizeof(SPVM_RUNTIME_STRING) / sizeof(int32_t)) * (SPVM_COMPILER_get_constant_strings_length(compiler));
-  
-  // class_vars_length
-  length++;
-  
-  // class_vars
-  length += (sizeof(SPVM_RUNTIME_CLASS_VAR) / sizeof(int32_t)) * (SPVM_COMPILER_get_class_vars_length(compiler));
-  
-  // fields_length
-  length++;
-  
-  // fields
-  length += (sizeof(SPVM_RUNTIME_FIELD) / sizeof(int32_t)) * (SPVM_COMPILER_get_fields_length(compiler));
-  
-  // methods_length
-  length++;
-  
-  // methods
-  length += (sizeof(SPVM_RUNTIME_METHOD) / sizeof(int32_t)) * (SPVM_COMPILER_get_methods_length(compiler));
-  
-  // args_length
-  length++;
-  
-  // args
-  length += (sizeof(SPVM_RUNTIME_ARG) / sizeof(int32_t)) * (SPVM_COMPILER_get_args_length(compiler));
-  
-  // opcodes_length
-  length++;
-  
-  // opcodes
-  length += (sizeof(SPVM_OPCODE) / sizeof(int32_t)) * (SPVM_COMPILER_get_opcodes_length(compiler));
-  
-  // anon_basic_types_length
-  length++;
-  
-  // anon_basic_types
-  length += (sizeof(int32_t) / sizeof(int32_t)) * (SPVM_COMPILER_get_anon_basic_types_length(compiler));
-  
-  return length;
-}
-
-int32_t* SPVM_COMPILER_create_runtime_codes(SPVM_COMPILER* compiler, SPVM_ALLOCATOR* allocator) {
-
   SPVM_COMPILER_compile(compiler, NULL);
   
-  int32_t runtime_codes_length = SPVM_COMPILER_calculate_runtime_codes_length(compiler);
-  int32_t* runtime_codes = SPVM_ALLOCATOR_alloc_memory_block_permanent(allocator, sizeof(int32_t) * runtime_codes_length);
-  
-  int32_t* runtime_codes_ptr = runtime_codes;
-  
-  // Total length
-  *runtime_codes_ptr = runtime_codes_length;
-  runtime_codes_ptr++;
-  
-  // basic_types length
-  *runtime_codes_ptr = compiler->basic_types->length;
-  runtime_codes_ptr++;
-  
-  // basic_types
-  int32_t string_pool_base = 0;
-  int32_t constant_strings_base = 0;
-  int32_t class_vars_base = 0;
-  int32_t fields_base = 0;
-  int32_t methods_base = 0;
-  int32_t args_base = 0;
-  int32_t opcodes_runtime_codes_base = 0;
-  int32_t opcodes_base = 0;
+  SPVM_RUNTIME_BASIC_TYPE* runtime_basic_types = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_BASIC_TYPE) * compiler->basic_types->length);
+  int32_t runtime_basic_types_length = compiler->basic_types->length;
   
   for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
     SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    SPVM_RUNTIME_BASIC_TYPE* runtime_basic_type = (SPVM_RUNTIME_BASIC_TYPE*)runtime_codes_ptr;
+    SPVM_RUNTIME_BASIC_TYPE* runtime_basic_type = &runtime_basic_types[basic_type_id];
+
+    // New logic
+    const char* runtime_string_pool = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, basic_type->string_pool->length);
+    memcpy((char*)runtime_string_pool, basic_type->string_pool->string, basic_type->string_pool->length);
+    runtime_basic_type->string_pool = runtime_string_pool;
+    runtime_basic_type->string_pool_length = basic_type->string_pool->length;
     
+    SPVM_RUNTIME_STRING* runtime_constant_strings = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_STRING) * basic_type->constant_strings->length);
+    for (int32_t constant_string_index = 0; constant_string_index < basic_type->constant_strings->length; constant_string_index++) {
+      SPVM_STRING* constant_string = SPVM_LIST_get(basic_type->constant_strings, constant_string_index);
+      SPVM_RUNTIME_STRING* runtime_constant_string = &runtime_constant_strings[constant_string_index];
+      runtime_constant_string->value = &runtime_basic_type->string_pool[constant_string->string_pool_index];
+      runtime_constant_string->length = constant_string->length;
+    }
+    runtime_basic_type->constant_strings = runtime_constant_strings;
+    runtime_basic_type->constant_strings_length = basic_type->constant_strings->length;
+    
+    // New logic
+    if (basic_type->class_vars->length > 0) {
+      SPVM_RUNTIME_CLASS_VAR* runtime_class_vars = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_CLASS_VAR) * basic_type->class_vars->length);
+      for (int32_t class_var_index = 0; class_var_index < basic_type->class_vars->length; class_var_index++) {
+        SPVM_CLASS_VAR* class_var = SPVM_LIST_get(basic_type->class_vars, class_var_index);
+        SPVM_RUNTIME_CLASS_VAR* runtime_class_var = &runtime_class_vars[class_var_index];
+        
+        runtime_class_var->index = class_var->index;
+        runtime_class_var->basic_type = &runtime_basic_types[class_var->type->basic_type->id];
+        runtime_class_var->type_dimension = class_var->type->dimension;
+        runtime_class_var->type_flag = class_var->type->flag;
+        runtime_class_var->current_basic_type = &runtime_basic_types[class_var->current_basic_type->id];
+        
+        SPVM_STRING* class_var_name_string = SPVM_HASH_get(basic_type->constant_string_symtable, class_var->name, strlen(class_var->name));
+        runtime_class_var->name = runtime_basic_type->constant_strings[class_var_name_string->index].value;
+      }
+      runtime_basic_type->class_vars = runtime_class_vars;
+      runtime_basic_type->class_vars_length = basic_type->class_vars->length;
+    }
+
+    if (basic_type->fields->length > 0) {
+      SPVM_RUNTIME_FIELD* runtime_fields = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_FIELD) * basic_type->fields->length);
+      for (int32_t field_index = 0; field_index < basic_type->fields->length; field_index++) {
+        SPVM_FIELD* field = SPVM_LIST_get(basic_type->fields, field_index);
+        SPVM_RUNTIME_FIELD* runtime_field = &runtime_fields[field_index];
+        
+        runtime_field->index = field->index;
+        runtime_field->offset = field->offset;
+        runtime_field->basic_type = &runtime_basic_types[field->type->basic_type->id];
+        runtime_field->type_dimension = field->type->dimension;
+        runtime_field->type_flag = field->type->flag;
+        runtime_field->current_basic_type = &runtime_basic_types[field->current_basic_type->id];
+        
+        SPVM_STRING* field_name_string = SPVM_HASH_get(basic_type->constant_string_symtable, field->name, strlen(field->name));
+        runtime_field->name = runtime_basic_type->constant_strings[field_name_string->index].value;
+      }
+      runtime_basic_type->fields = runtime_fields;
+      runtime_basic_type->fields_length = basic_type->fields->length;
+    }
+    
+    if (basic_type->methods->length > 0) {
+      SPVM_RUNTIME_METHOD* runtime_methods = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_METHOD) * basic_type->methods->length);
+      for (int32_t method_index = 0; method_index < basic_type->methods->length; method_index++) {
+        
+        SPVM_METHOD* method = SPVM_LIST_get(basic_type->methods, method_index);
+        SPVM_RUNTIME_METHOD* runtime_method = &runtime_methods[method_index];
+        
+        assert(method->opcode_list->length > 0);
+        
+        runtime_method->opcodes = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_OPCODE) * method->opcode_list->length);
+        memcpy(runtime_method->opcodes, method->opcode_list->values, sizeof(SPVM_OPCODE) * method->opcode_list->length);
+        runtime_method->opcodes_length = method->opcode_list->length;
+        
+        runtime_method->index = method->index;
+        runtime_method->current_basic_type = &runtime_basic_types[method->current_basic_type->id];
+        runtime_method->is_class_method = method->is_class_method;
+        runtime_method->is_init = method->is_init;
+        runtime_method->is_anon = method->is_anon;
+        runtime_method->byte_vars_width  = method->byte_vars_width;
+        runtime_method->short_vars_width  = method->short_vars_width;
+        runtime_method->int_vars_width  = method->int_vars_width;
+        runtime_method->long_vars_width  = method->long_vars_width;
+        runtime_method->float_vars_width  = method->float_vars_width;
+        runtime_method->double_vars_width  = method->double_vars_width;
+        runtime_method->object_vars_width = method->object_vars_width;
+        runtime_method->ref_vars_width = method->ref_vars_width;
+        runtime_method->mortal_stack_length  = method->mortal_stack_length;
+        runtime_method->return_basic_type = &runtime_basic_types[method->return_type->basic_type->id];
+        runtime_method->return_type_dimension = method->return_type->dimension;
+        runtime_method->return_type_flag = method->return_type->flag;
+        runtime_method->is_native = method->is_native;
+        runtime_method->is_precompile = method->is_precompile;
+        runtime_method->is_destructor = method->is_destructor;
+        runtime_method->is_required = method->is_required;
+        runtime_method->is_enum = method->is_enum;
+        
+        SPVM_STRING* method_name_string = SPVM_HASH_get(basic_type->constant_string_symtable, method->name, strlen(method->name));
+        runtime_method->name = runtime_basic_type->constant_strings[method_name_string->index].value;
+        
+        if (method->args_length > 0) {
+          runtime_method->args = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_ARG) * method->args_length);
+          runtime_method->args_length = method->args_length;
+          for (int32_t arg_index = 0; arg_index < method->args_length; arg_index++) {
+            SPVM_VAR_DECL* arg_var_decl = SPVM_LIST_get(method->var_decls, arg_index);
+            SPVM_RUNTIME_ARG* runtime_arg = &runtime_method->args[arg_index];
+            
+            runtime_arg->index = arg_index;
+            runtime_arg->basic_type = &runtime_basic_types[arg_var_decl->type->basic_type->id];
+            runtime_arg->type_dimension = arg_var_decl->type->dimension;
+            runtime_arg->type_flag = arg_var_decl->type->flag;
+          }
+        }
+        
+        runtime_method->required_args_length = method->required_args_length;
+      }
+      runtime_basic_type->methods = runtime_methods;
+      runtime_basic_type->methods_length = basic_type->methods->length;
+    }
+    
+    if (basic_type->anon_basic_types->length > 0) {
+      SPVM_RUNTIME_BASIC_TYPE* runtime_anon_basic_types = SPVM_ALLOCATOR_alloc_memory_block_permanent(runtime->allocator, sizeof(SPVM_RUNTIME_BASIC_TYPE) * basic_type->anon_basic_types->length);
+      
+      for (int32_t anon_basic_type_index = 0; anon_basic_type_index < basic_type->anon_basic_types->length; anon_basic_type_index++) {
+        SPVM_BASIC_TYPE* anon_basic_type = SPVM_LIST_get(basic_type->anon_basic_types, anon_basic_type_index);
+        runtime_anon_basic_types[anon_basic_type_index] = runtime_basic_types[anon_basic_type->id];
+      }
+      runtime_basic_type->anon_basic_types = runtime_anon_basic_types;
+      runtime_basic_type->anon_basic_types_length = basic_type->anon_basic_types->length;
+    }
+
+    // Old logic
     runtime_basic_type->id = basic_type->id;
     runtime_basic_type->category = basic_type->category;
     
     SPVM_STRING* basic_type_string = SPVM_HASH_get(basic_type->constant_string_symtable, basic_type->name, strlen(basic_type->name));
     assert(basic_type_string->index >= 0);
-    runtime_basic_type->name_string_index = basic_type_string->index;
+    runtime_basic_type->name = runtime_basic_type->constant_strings[basic_type_string->index].value;
     
     if (basic_type->module_rel_file) {
       SPVM_STRING* basic_type_rel_file_string = SPVM_HASH_get(basic_type->constant_string_symtable, basic_type->module_rel_file, strlen(basic_type->module_rel_file));
-      runtime_basic_type->module_rel_file_string_index = basic_type_rel_file_string->index;
-    }
-    else {
-      runtime_basic_type->module_rel_file_string_index = -1;
+      runtime_basic_type->module_rel_file = runtime_basic_type->constant_strings[basic_type_rel_file_string->index].value;
     }
     
     if (basic_type->module_dir) {
       SPVM_STRING* basic_type_dir_string = SPVM_HASH_get(basic_type->constant_string_symtable, basic_type->module_dir, strlen(basic_type->module_dir));
-      runtime_basic_type->module_dir_string_index = basic_type_dir_string->index;
+      runtime_basic_type->module_dir = runtime_basic_type->constant_strings[basic_type_dir_string->index].value;
     }
-    else {
-      runtime_basic_type->module_dir_string_index = -1;
+    
+    if (basic_type->version_string) {
+      SPVM_STRING* basic_type_version_string = SPVM_HASH_get(basic_type->constant_string_symtable, basic_type->version_string, strlen(basic_type->version_string));
+      runtime_basic_type->version_string = runtime_basic_type->constant_strings[basic_type_version_string->index].value;
     }
     
     runtime_basic_type->is_anon = basic_type->is_anon;
     
     runtime_basic_type->is_pointer = basic_type->is_pointer;
     
-    if (basic_type->parent_name) {
-      SPVM_BASIC_TYPE* parent_basic_type = SPVM_HASH_get(compiler->basic_type_symtable, basic_type->parent_name, strlen(basic_type->parent_name));
-      runtime_basic_type->parent_id = parent_basic_type->id;
-    }
-    else {
-      runtime_basic_type->parent_id = -1;
+    if (basic_type->parent) {
+      SPVM_BASIC_TYPE* parent_basic_type = SPVM_HASH_get(compiler->basic_type_symtable, basic_type->parent->name, strlen(basic_type->parent->name));
+      runtime_basic_type->parent = &runtime_basic_types[parent_basic_type->id];
     }
     
-    runtime_basic_type->fields_size = basic_type->fields_size;
-    
-    if (basic_type->version_string) {
-      SPVM_STRING* basic_type_version_string = SPVM_HASH_get(basic_type->constant_string_symtable, basic_type->version_string, strlen(basic_type->version_string));
-      runtime_basic_type->version_string_string_index = basic_type_version_string->index;
-    }
-    else {
-      runtime_basic_type->version_string_string_index = -1;
-    }
+    runtime_basic_type->fields_width = basic_type->fields_width;
     
     if (basic_type->init_method) {
-      runtime_basic_type->init_method_index = basic_type->init_method->index;
-    }
-    else {
-      runtime_basic_type->init_method_index = -1;
+      runtime_basic_type->init_method = &runtime_basic_type->methods[basic_type->init_method->index];
     }
     
     if (basic_type->destructor_method) {
-      runtime_basic_type->destructor_method_index = basic_type->destructor_method->index;
-    }
-    else {
-      runtime_basic_type->destructor_method_index = -1;
-    }
-    
-    if (basic_type->category == SPVM_NATIVE_C_BASIC_TYPE_CATEGORY_INTERFACE) {
-      assert(basic_type->required_method);
+      runtime_basic_type->destructor_method = &runtime_basic_type->methods[basic_type->destructor_method->index];
     }
     
     if (basic_type->required_method) {
-      runtime_basic_type->required_method_index = basic_type->required_method->index;
-    }
-    else {
-      runtime_basic_type->required_method_index = -1;
-    }
-    
-    runtime_basic_type->methods_length = basic_type->methods->length;
-    if (basic_type->methods->length > 0) {
-      runtime_basic_type->methods_base = methods_base;
-      methods_base += basic_type->methods->length;
-    }
-    else {
-      runtime_basic_type->methods_base = -1;
-    }
-    
-    runtime_basic_type->fields_length = basic_type->fields->length;
-    if (basic_type->fields->length > 0) {
-      runtime_basic_type->fields_base = fields_base;
-      fields_base += basic_type->fields->length;
-    }
-    else {
-      runtime_basic_type->fields_base = -1;
-    }
-    
-    runtime_basic_type->class_vars_length = basic_type->class_vars->length;
-    if (basic_type->class_vars->length > 0) {
-      runtime_basic_type->class_vars_base = class_vars_base;
-      class_vars_base += basic_type->class_vars->length;
-    }
-    else {
-      runtime_basic_type->class_vars_base = -1;
-    }
-    
-    runtime_basic_type->string_pool_length = SPVM_COMPILER_get_string_pool_length(compiler);
-    if (basic_type->string_pool->length > 0) {
-      runtime_basic_type->string_pool_base = string_pool_base;
-      string_pool_base += basic_type->string_pool->length;
-    }
-    else {
-      runtime_basic_type->string_pool_base = -1;
-    }
-    
-    runtime_basic_type->constant_strings_length = basic_type->constant_strings->length;
-    if (basic_type->constant_strings->length > 0) {
-      runtime_basic_type->constant_strings_base = constant_strings_base;
-      constant_strings_base += basic_type->constant_strings->length;
-    }
-    else {
-      runtime_basic_type->constant_strings_base = -1;
-    }
-    
-    runtime_codes_ptr += sizeof(SPVM_RUNTIME_BASIC_TYPE) / sizeof(int32_t);
-  }
-  
-  // string_pool_length
-  int32_t string_pool_length = SPVM_COMPILER_get_string_pool_length(compiler);
-  *runtime_codes_ptr = string_pool_length;
-  runtime_codes_ptr++;
-  
-  // string_pool
-  {
-    int32_t string_pool_base = 0;
-    for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-      SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-      memcpy((char*)((intptr_t)runtime_codes_ptr + string_pool_base), basic_type->string_pool->string, basic_type->string_pool->length);
-      string_pool_base += basic_type->string_pool->length;
-    }
-  }
-  runtime_codes_ptr += string_pool_length / sizeof(int32_t);
-  
-  // constant_strings_length
-  *runtime_codes_ptr = SPVM_COMPILER_get_constant_strings_length(compiler);
-  runtime_codes_ptr++;
-  
-  // constant_strings
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t constant_string_index = 0; constant_string_index < basic_type->constant_strings->length; constant_string_index++) {
-      SPVM_STRING* string = SPVM_LIST_get(basic_type->constant_strings, constant_string_index);
-      SPVM_RUNTIME_STRING* runtime_string = (SPVM_RUNTIME_STRING*)runtime_codes_ptr;
-      
-      runtime_string->index = string->index;
-      runtime_string->length = string->length;
-      runtime_string->string_pool_index = string->string_pool_index;
-      runtime_codes_ptr += sizeof(SPVM_RUNTIME_STRING) / sizeof(int32_t);
+      runtime_basic_type->required_method = &runtime_basic_type->methods[basic_type->required_method->index];
     }
   }
   
-  // class_vars_length
-  int32_t class_vars_length = SPVM_COMPILER_get_class_vars_length(compiler);
-  *runtime_codes_ptr = class_vars_length;
-  runtime_codes_ptr++;
+  // basic_types length
+  runtime->basic_types_length = runtime_basic_types_length;
   
-  // class_vars
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t class_var_index = 0; class_var_index < basic_type->class_vars->length; class_var_index++) {
-      SPVM_CLASS_VAR* class_var = SPVM_LIST_get(basic_type->class_vars, class_var_index);
-      SPVM_RUNTIME_CLASS_VAR* runtime_class_var = (SPVM_RUNTIME_CLASS_VAR*)runtime_codes_ptr;
-      
-      runtime_class_var->index = class_var->index;
-      runtime_class_var->basic_type_id = class_var->type->basic_type->id;
-      runtime_class_var->type_dimension = class_var->type->dimension;
-      runtime_class_var->type_flag = class_var->type->flag;
-      runtime_class_var->current_basic_type_id = class_var->current_basic_type->id;
-      
-      SPVM_STRING* class_var_name_string = SPVM_HASH_get(basic_type->constant_string_symtable, class_var->name, strlen(class_var->name));
-      runtime_class_var->name_string_index = class_var_name_string->index;
-      
-      runtime_codes_ptr += sizeof(SPVM_RUNTIME_CLASS_VAR) / sizeof(int32_t);
-    }
+  // basic_types
+  runtime->basic_types = runtime_basic_types;
+  
+  // Runtime basic type symtable
+  runtime->basic_type_symtable = SPVM_HASH_new_hash_permanent(runtime->allocator, runtime->basic_types_length);
+  for (int32_t basic_type_id = 0; basic_type_id < runtime->basic_types_length; basic_type_id++) {
+    SPVM_RUNTIME_BASIC_TYPE* basic_type = &runtime->basic_types[basic_type_id];
+    SPVM_HASH_set(runtime->basic_type_symtable, basic_type->name, strlen(basic_type->name), basic_type);
   }
   
-  // fields_length
-  int32_t fields_length = SPVM_COMPILER_get_fields_length(compiler);
-  *runtime_codes_ptr = fields_length;
-  runtime_codes_ptr++;
-  
-  // fields
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t field_index = 0; field_index < basic_type->fields->length; field_index++) {
-      SPVM_FIELD* field = SPVM_LIST_get(basic_type->fields, field_index);
-      SPVM_RUNTIME_FIELD* runtime_field = (SPVM_RUNTIME_FIELD*)runtime_codes_ptr;
-      
-      runtime_field->index = field->index;
-      runtime_field->offset = field->offset;
-      runtime_field->basic_type_id = field->type->basic_type->id;
-      runtime_field->type_dimension = field->type->dimension;
-      runtime_field->type_flag = field->type->flag;
-      runtime_field->current_basic_type_id = field->current_basic_type->id;
-      
-      SPVM_STRING* field_name_string = SPVM_HASH_get(basic_type->constant_string_symtable, field->name, strlen(field->name));
-      runtime_field->name_string_index = field_name_string->index;
-      
-      runtime_codes_ptr += sizeof(SPVM_RUNTIME_FIELD) / sizeof(int32_t);
-    }
-  }
-  
-  // methods_length
-  int32_t methods_length = SPVM_COMPILER_get_methods_length(compiler);
-  *runtime_codes_ptr = methods_length;
-  runtime_codes_ptr++;
-  
-  // methods
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t method_index = 0; method_index < basic_type->methods->length; method_index++) {
-      
-      SPVM_METHOD* method = SPVM_LIST_get(basic_type->methods, method_index);
-      SPVM_RUNTIME_METHOD* runtime_method = (SPVM_RUNTIME_METHOD*)runtime_codes_ptr;
-      
-      SPVM_OPCODE_LIST* opcode_list = method->opcode_list;
-      runtime_method->opcodes_base = opcodes_base;
-      runtime_method->opcodes_length = opcode_list->length;
-      opcodes_base += opcode_list->length;
-      
-      runtime_method->index = method->index;
-      runtime_method->current_basic_type_id = method->current_basic_type->id;
-      runtime_method->is_class_method = method->is_class_method;
-      runtime_method->is_init = method->is_init;
-      runtime_method->is_anon = method->is_anon;
-      runtime_method->call_stack_byte_vars_length  = method->call_stack_byte_vars_length;
-      runtime_method->call_stack_short_vars_length  = method->call_stack_short_vars_length;
-      runtime_method->call_stack_int_vars_length  = method->call_stack_int_vars_length;
-      runtime_method->call_stack_long_vars_length  = method->call_stack_long_vars_length;
-      runtime_method->call_stack_float_vars_length  = method->call_stack_float_vars_length;
-      runtime_method->call_stack_double_vars_length  = method->call_stack_double_vars_length;
-      runtime_method->call_stack_object_vars_length = method->call_stack_object_vars_length;
-      runtime_method->call_stack_ref_vars_length = method->call_stack_ref_vars_length;
-      runtime_method->mortal_stack_length  = method->mortal_stack_length;
-      runtime_method->return_basic_type_id = method->return_type->basic_type->id;
-      runtime_method->return_type_dimension = method->return_type->dimension;
-      runtime_method->return_type_flag = method->return_type->flag;
-      runtime_method->is_native = method->is_native;
-      runtime_method->is_precompile = method->is_precompile;
-      runtime_method->is_destructor = method->is_destructor;
-      runtime_method->is_required = method->is_required;
-      runtime_method->is_enum = method->is_enum;
-      
-      SPVM_STRING* method_name_string = SPVM_HASH_get(basic_type->constant_string_symtable, method->name, strlen(method->name));
-      runtime_method->name_string_index = method_name_string->index;
-      
-      runtime_method->args_length = method->args_length;
-      if (method->args_length > 0) {
-        runtime_method->args_base = args_base;
-        args_base += method->args_length;
-      }
-      else {
-        runtime_method->args_base = -1;
-      }
-      runtime_method->required_args_length = method->required_args_length;
-      
-      runtime_codes_ptr += sizeof(SPVM_RUNTIME_METHOD) / sizeof(int32_t);
-    }
-  }
-  
-  // args_length
-  int32_t args_length = SPVM_COMPILER_get_args_length(compiler);
-  *runtime_codes_ptr = args_length;
-  runtime_codes_ptr++;
-  
-  // args
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t method_index = 0; method_index < basic_type->methods->length; method_index++) {
-      
-      SPVM_METHOD* method = SPVM_LIST_get(basic_type->methods, method_index);
-      for (int32_t arg_index = 0; arg_index < method->args_length; arg_index++) {
-        SPVM_VAR_DECL* arg_var_decl = SPVM_LIST_get(method->var_decls, arg_index);
-        SPVM_RUNTIME_ARG* runtime_arg = (SPVM_RUNTIME_ARG*)runtime_codes_ptr;
-        
-        runtime_arg->index = arg_index;
-        runtime_arg->basic_type_id = arg_var_decl->type->basic_type->id;
-        runtime_arg->type_dimension = arg_var_decl->type->dimension;
-        runtime_arg->type_flag = arg_var_decl->type->flag;
-        
-        runtime_codes_ptr += sizeof(SPVM_RUNTIME_ARG) / sizeof(int32_t);
-      }
-    }
-  }
-  
-  // opcodes_length
-  int32_t opcodes_length = SPVM_COMPILER_get_opcodes_length(compiler);
-  *runtime_codes_ptr = opcodes_length;
-  runtime_codes_ptr++;
-  
-  // opcodes
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t method_index = 0; method_index < basic_type->methods->length; method_index++) {
-      SPVM_METHOD* method = SPVM_LIST_get(basic_type->methods, method_index);
-      SPVM_OPCODE_LIST* opcode_list = method->opcode_list;
-      memcpy(runtime_codes_ptr, opcode_list->values, sizeof(SPVM_OPCODE) * opcode_list->length);
-      runtime_codes_ptr += (sizeof(SPVM_OPCODE) / sizeof(int32_t)) * opcode_list->length;
-    }
-  }
-  
-  // anon_basic_types_length
-  int32_t anon_basic_types_length = SPVM_COMPILER_get_anon_basic_types_length(compiler);
-  *runtime_codes_ptr = anon_basic_types_length;
-  runtime_codes_ptr++;
-  
-  // anon_basic_type_ids
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t anon_basic_type_index = 0; anon_basic_type_index < basic_type->anon_basic_types->length; anon_basic_type_index++) {
-      SPVM_BASIC_TYPE* anon_basic_type = SPVM_LIST_get(basic_type->anon_basic_types, anon_basic_type_index);
-      *runtime_codes_ptr = anon_basic_type->id;
-      runtime_codes_ptr += sizeof(int32_t) / sizeof(int32_t);
-    }
-  }
-  
-  return runtime_codes;
+  return runtime;
 }
 
 void SPVM_COMPILER_error(SPVM_COMPILER* compiler, const char* message_template, ...) {
@@ -1058,97 +850,3 @@ const char* SPVM_COMPILER_get_include_dir (SPVM_COMPILER* compiler, int32_t incl
   const char* include_dir = SPVM_LIST_get(compiler->include_dirs, include_dir_id);
   return include_dir;
 }
-
-int32_t SPVM_COMPILER_get_class_vars_length(SPVM_COMPILER* compiler) {
-  
-  int32_t class_vars_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    class_vars_length += basic_type->class_vars->length;
-  }
-  
-  return class_vars_length;
-}
-
-int32_t SPVM_COMPILER_get_fields_length(SPVM_COMPILER* compiler) {
-  
-  int32_t fields_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    fields_length += basic_type->fields->length;
-  }
-  
-  return fields_length;
-}
-
-int32_t SPVM_COMPILER_get_methods_length(SPVM_COMPILER* compiler) {
-  
-  int32_t methods_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    methods_length += basic_type->methods->length;
-  }
-  
-  return methods_length;
-}
-
-int32_t SPVM_COMPILER_get_anon_basic_types_length(SPVM_COMPILER* compiler) {
-  
-  int32_t anon_basic_types_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    anon_basic_types_length += basic_type->anon_basic_types->length;
-  }
-  
-  return anon_basic_types_length;
-}
-
-int32_t SPVM_COMPILER_get_args_length(SPVM_COMPILER* compiler) {
-  
-  int32_t args_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t method_index = 0;method_index < basic_type->methods->length; method_index++) {
-      SPVM_METHOD* method = SPVM_LIST_get(basic_type->methods, method_index);
-      args_length += method->args_length;
-    }
-  }
-  
-  return args_length;
-}
-
-int32_t SPVM_COMPILER_get_opcodes_length(SPVM_COMPILER* compiler) {
-  
-  int32_t opcodes_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    for (int32_t method_index = 0;method_index < basic_type->methods->length; method_index++) {
-      SPVM_METHOD* method = SPVM_LIST_get(basic_type->methods, method_index);
-      opcodes_length += method->opcode_list->length;
-    }
-  }
-  
-  return opcodes_length;
-}
-
-int32_t SPVM_COMPILER_get_constant_strings_length(SPVM_COMPILER* compiler) {
-  
-  int32_t constant_strings_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    constant_strings_length += basic_type->constant_strings->length;
-  }
-  
-  return constant_strings_length;
-}
-
-int32_t SPVM_COMPILER_get_string_pool_length(SPVM_COMPILER* compiler) {
-  
-  int32_t string_pool_length = 0;
-  for (int32_t basic_type_id = 0; basic_type_id < compiler->basic_types->length; basic_type_id++) {
-    SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-    string_pool_length += basic_type->string_pool->length;
-  }
-  
-  return string_pool_length;
-}
