@@ -1,3 +1,6 @@
+// Copyright (c) 2023 Yuki Kimoto
+// MIT License
+
 #include "spvm_string.h"
 #include "spvm_allocator.h"
 #include "spvm_compiler.h"
@@ -12,17 +15,11 @@ SPVM_STRING* SPVM_STRING_new(SPVM_COMPILER* compiler, const char* value, int32_t
     return found_string;
   }
   else {
-    int32_t string_buffer_id = compiler->string_buffer->length;
-    
-    SPVM_STRING_BUFFER_add_len_nullstr(compiler->string_buffer, (char*)value, length);
-    
     SPVM_STRING* string = SPVM_ALLOCATOR_alloc_memory_block_permanent(compiler->allocator, sizeof(SPVM_STRING));
-    string->value = (char*)(compiler->string_buffer->value + string_buffer_id);
+    string->value = SPVM_ALLOCATOR_alloc_memory_block_permanent(compiler->allocator, length + 1);
+    memcpy((char*)string->value, value, length);
     string->length = length;
-    string->id = compiler->strings->length;
-    string->string_buffer_id = string_buffer_id;
     
-    SPVM_LIST_push(compiler->strings, string);
     SPVM_HASH_set(compiler->string_symtable, string->value, length, string);
     
     return string;
