@@ -8,10 +8,7 @@ SPVM::Document::NativeAPI::Runtime - SPVM Runtime Native APIs
   SSPVM_ENV_RUNTIME* runtime_api = env->api->runtime;
   
   // New a runtime object
-  SPVM_ENV_RUNTIME* runtime = runtime_api->new_instance();
-  
-  // Build the runtime
-  env->api->runtime->build(runtime, runtime_codes);
+  void* runtime = runtime_api->new_instance();
   
   // Free a runtime object
   env->api->runtime->free_instance(runtime);
@@ -28,8 +25,8 @@ Runtime native APIs have its IDs.
   1  free_instance
   2  get_opcodes
   3  reserved3
-  4  get_runtime_codes
-  5  get_runtime_codes_length
+  4  reserved4
+  5  reserved5
   6  reserved6
   7  reserved7
   8  reserved8
@@ -74,18 +71,18 @@ Runtime native APIs have its IDs.
   47 get_method_is_anon
   48 get_method_is_native
   49 get_method_is_precompile
-  50 get_method_call_stack_byte_vars_length
-  51 get_method_call_stack_short_vars_length
-  52 get_method_call_stack_int_vars_length
-  53 get_method_call_stack_long_vars_length
-  54 get_method_call_stack_float_vars_length
-  55 get_method_call_stack_double_vars_length
-  56 get_method_call_stack_object_vars_length
-  57 get_method_call_stack_ref_vars_length
+  50 get_method_byte_vars_width
+  51 get_method_short_vars_width
+  52 get_method_int_vars_width
+  53 get_method_long_vars_width
+  54 get_method_float_vars_width
+  55 get_method_double_vars_width
+  56 get_method_object_vars_width
+  57 get_method_ref_vars_width
   58 get_method_mortal_stack_length
-  59 get_method_opcodes_base
+  59 reserved59
   60 get_method_opcodes_length
-  61 get_method_args_base
+  61 reserved61
   62 get_method_args_length
   63 reserved63
   64 reserved64
@@ -100,8 +97,8 @@ Runtime native APIs have its IDs.
   73 reserved73
   74 reserved74
   75 object_length_offset
-  76 get_allocator
-  77 build
+  76 reserved76
+  77 reserved77
   78 reserved78
   79 get_method_required_args_length
   80 reserved80
@@ -114,11 +111,11 @@ Runtime native APIs have its IDs.
   87 get_basic_type_module_rel_file
   88 get_basic_type_module_dir
   89 get_basic_type_is_anon
-  90 get_basic_type_fields_base
+  90 reserved90
   91 get_basic_type_fields_length
-  92 get_basic_type_methods_base
+  92 reserved92
   93 get_basic_type_methods_length
-  94 get_basic_type_class_vars_base
+  94 reserved94
   95 get_basic_type_class_vars_length
   96 reserved96
   97 reserved97
@@ -174,14 +171,6 @@ Runtime native APIs have its IDs.
 
   int32_t* (*get_opcodes)(void* runtime);
 
-=head2 get_runtime_codes
-
-  int32_t* (*get_runtime_codes)(void* runtime);
-
-=head2 get_runtime_codes_length
-
-  int32_t (*get_runtime_codes_length)(void* runtime);
-
 =head2 get_basic_type_id_by_name
 
   int32_t (*get_basic_type_id_by_name)(void* runtime, const char* basic_type_name);
@@ -206,26 +195,14 @@ Runtime native APIs have its IDs.
 
   int32_t (*get_basic_type_is_anon)(void* runtime, void* basic_type);
 
-=head2 get_basic_type_fields_base
-
-  int32_t (*get_basic_type_fields_base)(void* runtime, void* basic_type);
-
 =head2 get_basic_type_fields_length
 
   int32_t (*get_basic_type_fields_length)(void* runtime, void* basic_type);
 
-=head2 get_basic_type_methods_base
-
-  int32_t (*get_basic_type_methods_base)(void* runtime, void* basic_type);
-
 =head2 get_basic_type_methods_length
 
   int32_t (*get_basic_type_methods_length)(void* runtime, void* basic_type);
 
-=head2 get_basic_type_class_vars_base
-
-  int32_t (*get_basic_type_class_vars_base)(void* runtime, void* basic_type);
-
 =head2 get_basic_type_class_vars_length
 
   int32_t (*get_basic_type_class_vars_length)(void* runtime, void* basic_type);
